package org.elsysbg.courses.ip.jpa.biology.converter;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.elsysbg.courses.ip.jpa.biology.model.AbstractEntity;
import org.elsysbg.courses.ip.jpa.biology.model.PersistenceManager;

/**
 * @author NIE
 * Converts from Entity to string using ID of the object in the DB
 */
abstract public class AbstractEntityConverter implements Converter {
	private static final Log LOGGER = LogFactory.getLog(AbstractEntityConverter.class);
	final private Class<? extends AbstractEntity> type;

	protected AbstractEntityConverter(final Class<? extends AbstractEntity> type) {
		this.type = type;
	}
	
	public Object getAsObject(final FacesContext context, final UIComponent component, final String value) throws ConverterException{
		if(value==null) {
			return null;
		}
		
		final long id;
		try {
			//read ID
			id = Long.parseLong(value);
		} catch (NumberFormatException e) {
			final String message = "unable to convert "+value+" to long (id)";
			LOGGER.error(message, e);
			throw new ConverterException(message, e); 
		}
		
		//get object behind this ID
		final AbstractEntity entity = PersistenceManager.getInstance().getEntityById(type, id);
		if(entity==null) {
			final String message = "unable to get entity with id "+value+": "+type;
			LOGGER.error(message);
			throw new ConverterException(message);			
		}
		return entity;
	}

	public String getAsString(final FacesContext context, final UIComponent component,
			final Object value) throws ConverterException {
		if (!(value instanceof AbstractEntity)) {
			final String message = "object is not abstract entity: "+value;
			LOGGER.error(message);
			throw new ConverterException(message);
		}

		final AbstractEntity entity = (AbstractEntity) value;
		return Long.toString(entity.getId());
	}
}