package org.elsysbg.courses.ip.jpa.biology.model;

import java.util.ArrayList;
import java.util.List;

import javax.faces.model.SelectItem;
import javax.persistence.Entity;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToMany;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Transient;

import org.elsysbg.courses.ip.jpa.biology.helper.CommonHelper;

@Entity
@Inheritance(strategy=InheritanceType.JOINED)
@NamedQueries({
	@NamedQuery(
			name="getAnimals",//get all animals
			query="SELECT a FROM Animal a"
			),
})
public class Animal extends AbstractEntity {

	protected static enum AnimalType {
		UNDEFINED,
		BIRD,
		REPTILE,
		STUDENT;
	}
	
	public Animal() {
		this(AnimalType.UNDEFINED);
	}

	public Animal(final AnimalType type) {
		this.type = type;
	}

	private String name;
	
	@ManyToMany
	private List<Food> foods = new ArrayList<Food>();
	
	@Transient
	private AnimalType type;
	
	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

	synchronized public void setFoods(List<Food> foods) {
		this.foods = foods;
	}

	synchronized public List<Food> getFoods() {
		return foods;
	}
	
	public boolean isBird() {
		return type == AnimalType.BIRD;
	}
	public boolean isReptile() {
		return type == AnimalType.REPTILE;
	}
	public List<SelectItem> getListAvailableFoods() {
		final List<Food> result = PersistenceManager.getInstance().getFoods();
		result.removeAll(getFoods());
		return CommonHelper.toSelectItemList(result);
	}
	
	public List<SelectItem> getListFoods() {
		return CommonHelper.toSelectItemList(getFoods());
	}
	
	@Override
	public String getLabel() {
		return name;
	}
	
	synchronized public boolean addFood(Food food) {
		if(foods.contains(food)) {
			return false;
		}
		
		foods.add(food);
		
		food.addAnimal(this);
		
		return true;
	}

	synchronized public boolean removeFood(Food food) {
		if(!foods.contains(food)) {
			return false;
		}
		
		foods.remove(food);
		
		food.removeAnimal(this);
		
		return true;
	}
	
}
