package org.elsysbg.courses.ip.jpa.biology.model;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.context.FacesContext;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.elsysbg.courses.ip.jpa.biology.application.ManagedBeanKeys;
import org.elsysbg.courses.ip.jpa.biology.helper.CommonHelper;

public class PersistenceManager {

	private static final Log LOGGER = LogFactory.getLog(PersistenceManager.class);

	static private PersistenceManager instance;
	
	private EntityManagerFactory emf;
	private EntityManager em;

	public PersistenceManager() {
		if(instance!=null) {
			throw new IllegalStateException("PersistenceManager must be singleton (no two instances may exists)");
		}
		instance = this;

		//configure path to DB
		final Map<String, String> dbProperties = new HashMap<String, String>();

		final File loginJSPFile = new File(((ServletContext) FacesContext.getCurrentInstance().getExternalContext().getContext())
		        .getRealPath("/"));
		final String dbPath = String.format("%s%cWEB-INF%cdb", loginJSPFile.getAbsolutePath(), File.separatorChar, File.separatorChar);
		final String connectionUrl = String.format("jdbc:derby:%s;create=true", dbPath);
		LOGGER.info("using connection url: "+connectionUrl);
		dbProperties.put("hibernate.connection.url", connectionUrl);
		
		if(!(new File(dbPath).exists())) {
			//create DB if directory does not exist
			LOGGER.warn("DB does not exist! Creating new...");
			dbProperties.put("hibernate.hbm2ddl.auto", "create");
		}
		
		emf = Persistence.createEntityManagerFactory("Biology", dbProperties);
		em = emf.createEntityManager();
	}
	
	static public PersistenceManager getInstance() {
		if (instance == null) {
			new PersistenceManager();
			//instance = this; in constructor
			CommonHelper.putApplicationMap(ManagedBeanKeys.persistenceManager, instance);
		}
		return instance;
	}
	
	synchronized public boolean addEntity(AbstractEntity entity) {
		final EntityTransaction tx = em.getTransaction();
		
		try {
		    tx.begin();

		    em.persist(entity);

		    tx.commit();
			LOGGER.debug("entity persisted: "+entity);
			return true;
		} catch (final RuntimeException e) {
			LOGGER.error("could not persist: "+entity, e);
			return false;
		} finally {
		    if (tx.isActive()) {
		        tx.rollback();
		        
		        //em.clear();
		    }

		}
	}
	
	synchronized public boolean removeEntity(AbstractEntity entity) {
		final EntityTransaction tx = em.getTransaction();
		
		try {
		    tx.begin();

		    em.remove(entity);

		    tx.commit();
			LOGGER.debug("entity removed: "+entity);
			return true;
		} catch (final RuntimeException e) {
			LOGGER.error("could not remove: "+entity, e);
			return false;
		} finally {
		    if (tx.isActive()) {
		        tx.rollback();
		        
		        //em.clear();
		    }

		}
	}	

	@SuppressWarnings("unchecked")
	synchronized public List<Animal> getAnimals() {
		return (List<Animal>)em.createNamedQuery("getAnimals").getResultList();
	}
	@SuppressWarnings("unchecked")
	synchronized public List<Food> getFoods() {
		return (List<Food>)em.createNamedQuery("getFoods").getResultList();
	}

	synchronized public <T extends AbstractEntity> T getEntityById(final Class<T> t, final long id) {
		final T result = em.find(t, id);
		if(result != null) {
			LOGGER.debug("getEntityById: "+result);
		} else {
			LOGGER.error("no entity with this id: "+id+" (type: "+t+")");
		}

		return result;
	}
}
